from . .queries import *
from . .regression import *
from statsmodels.sandbox.regression.predstd import wls_prediction_std
import matplotlib.pyplot as plt
import pandas as pd
import time

def q_drift(log_file_path):
    """
    Ask the user for their choice of correction and keep asking until a valid response is given.
    """
    while True:  # Start an infinite loop
        choice = input("\nApply drift to correction? (Y/N)\n")

        if choice in ["yes", "y", "true", "t", "no", "n", "false", "f"]:
            return choice
            append_to_log(log_file_path, f"Drift application: {choice}")
        else:
            print("\nInvalid response\n")
            
def process_drift_correction(samp,lin_std, drift_std, correction_log, fig_path, log_file_path, isotope):
    user_choice = q_drift(log_file_path)
    chains_to_correct = []
    if neg_response(user_choice): dD_temp = "dD"
    else:
        result = drift_correction(samp, lin_std, drift_std, fig_path, isotope, log_file_path, fig=True)
        if result is not None:
            samp, lin_std, drift_std = result  # Apply the correction
            dD_temp = "drift_corrected_dD"
            correction_log.loc["Drift", "samples"] = 1
        else:
            print("Correction not applied. Reverting to original data.")
            dD_temp = "dD" 
            # Ensure drift_corrected_dD column is created
            samp['drift_corrected_dD'] = samp['dD']
            lin_std['drift_corrected_dD'] = ['dD']
            drift_std['drift_corrected_dD'] = drift_std['dD']
    time.sleep(0)  # Wait for 1 second
    #clear_output(wait=True)
    return samp, lin_std, drift_std, dD_temp, correction_log
    

def drift_correction(samp, lin_std, drift_std, fig_path, isotope, log_file_path, fig=True):
    lin_std = lin_std.copy()
    drift_std = drift_std.copy()
    fig, axs = plt.subplots(1, 2, figsize=(5, 3))
    # Normalize chains
    norm=pd.DataFrame()
    mean_isotope_dict={}
    for i in ["C18","C24"]:
        mask = drift_std.chain==i
        temp = drift_std.loc[mask, [isotope, 'time_rel','chain']].copy()
        mean_isotope_dict[i] = temp[isotope].mean()
        temp[isotope] -= temp[isotope].mean() # center the data at zero
        norm = pd.concat([norm, temp])
    # Perform robust linear regression
    m_d, b_d, r_squared, p_value, std_error, model = wls_regression(norm['time_rel'], norm[isotope], log_file_path)
    new_time_rel = sm.add_constant(np.array(drift_std.time_rel))
    results = model.predict(new_time_rel)
    prstd, pred_ci_lower, pred_ci_upper = wls_prediction_std(model)#, alpha=0.05)
    f3_drift_times = drift_std['time_rel'].unique()[:3]                                             # Find last three samples
    relative_mean = np.mean(model.predict(sm.add_constant(drift_std[drift_std["time_rel"].isin(f3_drift_times)].time_rel))) # estimated dD of last three samples
    drift_correction = results - relative_mean # difference between estimated drift times and lowest replicate mean 
    # Apply drift correction to the drift standards themselves
    drift_std["drift_corrected_dD"] = drift_std[isotope] - drift_correction
    drift_std["drift_error"]        = prstd

    # Plotting if required
    #fig, axs = plt.subplots(1, 2, figsize=(5, 3))
    x = 0
    for i in ["C18","C24"]:
        # Predict values for plotting the regression line
        temp = drift_std[drift_std.chain==i]
        axs[x].scatter(temp['time_rel'], temp[isotope], alpha=0.4, ec='k', s=80, c='blue', label="Drift std")
        axs[x].scatter(temp['time_rel'], temp['drift_corrected_dD'], alpha=0.7, marker='x', s=80, c='red', label="Corrected drift std")
        # Plot regression line
        xmin, xmax = temp['time_rel'].min(), temp['time_rel'].max()
        axs[x].plot([xmin, xmax], [((xmin * m_d + b_d)+temp[temp.time_rel==xmin][isotope].mean()), (xmax * m_d + b_d+temp[temp.time_rel==xmax][isotope].mean())], c='k', linestyle='--', alpha=0.6) # Generalized least-squares regression
        axs[x].set_xlabel("Time relative") 
        axs[x].text(0.9, 0.9, i,
                  horizontalalignment='center', verticalalignment='center',
                  transform=axs[x].transAxes)
        if isotope == "dD": label = "δD (‰)"
        else: label = "δC (‰)"
        axs[x].set_ylabel(label)
        if x == 1:
            axs[x].legend(loc='upper center', bbox_to_anchor=(0, -0.2), fancybox=False, shadow=True,frameon=False, ncol=1)
        x= x+1
        
    # Show the plot before asking for confirmation
    plt.tight_layout()
    #plt.savefig(os.path.join(fig_path, 'Drift correction.png'), dpi=300, bbox_inches='tight')
    plt.show()
    
    # Print regression statistics for each chain
    print("\nRegression statistics drift standards:")
    print("Linear equation: "+str(label)+f" = ({m_d:.2f})(time) + {b_d:.2f}")
    print(f"Adjusted R²: {r_squared:.2f}")
    print(f"P-value: {p_value:.2f}")
    print(f"Standard Error: {std_error:.2f}")

    # Ask user for confirmation
    user_confirmation = input("Apply the correction? (Y/N): \n")
    if pos_response(user_confirmation):
        append_to_log(log_file_path, "Drift correction equation: δD = (time)("+str(m_d)+"+"+str(b_d)+"\nr2 = "+str(r_squared)+"\np = "+str(p_value))
        append_to_log(log_file_path, "User applied drift correction.")
        # Correct linearity standards
        new_time_rel = sm.add_constant(np.array(lin_std.time_rel))
        results = model.predict(new_time_rel)
        prstd, pred_ci_lower, pred_ci_upper = wls_prediction_std(model, exog=new_time_rel)#, alpha=0.05)
        lin_est_corr = results - relative_mean # offset between estimates of linearity samples based on time and the time_zero estimate
        lin_std['drift_corrected_dD'] = lin_std[isotope]-lin_est_corr
        lin_std['drift_error']        = prstd
        
        # Correct samples 
        new_time_rel = sm.add_constant(np.array(samp.time_rel))
        results = model.predict(new_time_rel)
        prstd, pred_ci_lower, pred_ci_upper = wls_prediction_std(model, exog =new_time_rel)#, alpha=0.05)
        samp_est_corr = results - relative_mean # offset between estimates of linearity samples based on time and the time_zero estimate
        samp_est_corr = results-relative_mean
        samp['drift_corrected_dD'] = samp[isotope]-samp_est_corr
        samp['drift_error']        = prstd
        return samp, lin_std, drift_std
    else:
        return None



import matplotlib.pyplot as plt
import os
from .regression import *


def std_plot(lin, drift, folder_path, fig_path, isotope, cutoff_line=None, regress=False, dD = "dD"):
    """
    Function to plot linearity and drift standards.
    ~GAO~12/1/2023
    """
    fig, axes = plt.subplots(2, 2, figsize=[6, 4], sharex=False)
    chain_length = ["C18", "C20", "C24", "C28", "C30"]
    ax = axes.flatten()

    for i in [0, 2]:
        temp = drift[drift.chain == chain_length[i]]
        ax[i].scatter(temp["date-time_true"], temp[dD], alpha=0.4, ec='k', s=80, c='blue')
        ax[i].text(0.9, 0.9, chain_length[i],
                   horizontalalignment='center', verticalalignment='center',
                   transform=ax[i].transAxes)

        temp = lin[lin.chain == chain_length[i + 1]]
        ax[i + 1].scatter(temp["area"], temp[dD], alpha=0.2, ec='k', s=80, c='orange')
        ax[i + 1].text(0.9, 0.9, chain_length[i + 1],
                      horizontalalignment='center', verticalalignment='center',
                      transform=ax[i + 1].transAxes)
    # Set x-axis labels for the third subplot (ax2)
    ax[2].set_xlabel('Date (mm-dd-yyyy)')
    ax[2].set_xticks(ax[2].get_xticks())
    ax[2].set_xticklabels(labels=ax[2].get_xticklabels(), rotation=45)
    ax[0].set_xticks([]);#ax[1].set_xticks([])

    ax[3].set_xlabel('Peak Area (mVs)')
    ax[0].set_title("Drift Standards")
    ax[1].set_title("Linearity Standards")
    if isotope == "dD": label = "δD"
    else: label = "δC"
    fig.supylabel('Normalized '+str(label)+' (‰)')

    # Plot user-defined cutoff line
    if cutoff_line is not None:
        ax[1].axvline(cutoff_line[0], c='red', linestyle='--')
        ax[3].axvline(cutoff_line[1], c='red', linestyle='--')
    # plt.show(block=True)
    plt.savefig(os.path.join(fig_path, 'Standards Raw.png'), dpi=300, bbox_inches='tight')
    plt.show()
    
def verify_lin_plot(lin, fig_path, dD_id, log_file_path,cutoff_line, isotope, regress=False):
    """
    Function to plot linearity and drift standards with color differentiation based on cutoff.
    ~GAO~12/4/2023
    """
    cutoff_line=float(cutoff_line)
    fig = plt.figure(figsize=[5, 3])
    above_cutoff = lin[lin["area"] >= cutoff_line]
    plt.scatter(above_cutoff["area"], above_cutoff[dD_id], alpha=0.4, ec='k', s=80, c='orange', label = "Above peak area threshold.")
    below_cutoff = lin[lin["area"] < cutoff_line]
    plt.axvline(cutoff_line,color='red',linestyle="--")
    plt.scatter(below_cutoff["area"], below_cutoff[dD_id], alpha=0.4, ec='k', s=80, c='grey', label = "Below peak area threshold.")
    if isotope == "dD": label = "δD"
    else: label = "δC"
    plt.ylabel("Normalized "+str(label)+" (‰)")
    plt.xlabel('Peak Area (mVs)')
    
    temp = lin[lin.area > cutoff_line]   
    slope, intercept, r_squared, p_value, std_error, model = wls_regression(temp['area'], temp[dD_id],log_file_path)
    plt.plot([temp.area.min(), temp.area.max()], [temp.area.min() * slope + intercept,
                                                           temp.area.max() * slope + intercept], c='k', linestyle='--')
    plt.legend(loc='upper center', bbox_to_anchor=(0.5, -0.2), frameon=False, fancybox=False, shadow=True, ncol=2)
    
    # Plot logarithmic best-fit line
    plt.savefig(os.path.join(fig_path, 'Linearity.png'), bbox_inches='tight')
    plt.show()
    print("\nRegression statistics drift standards:")
    print(f"Linear equation: {label} = ({slope:.2f})(time) + {intercept:.2f}")
    print(f"Adjusted R²: {r_squared:.2f}")
    print(f"P-value: {p_value:.2f}")
    print(f"Standard Error: {std_error:.2f}")        
      
def total_dD_correction_plot(uncorrected_unknown, unknown , folder_path, fig_path, isotope):
    unique_chains = unknown['Chain Length'].unique()
    num_chains    = len(unique_chains)
    if isotope == "dD": label = "δD"
    else: label = "δC"
    if num_chains>1:
        fig, axes     = plt.subplots(num_chains, 1, figsize=(5,3 * num_chains))  
        for i, chain in enumerate(unique_chains):
            chain_unknown             = unknown[unknown['Chain Length'] == chain]
            uncorrected_chain_unknown = uncorrected_unknown[uncorrected_unknown.Component == chain]
            if "Raw "+str(isotope) in uncorrected_chain_unknown:
                axes[i].scatter(uncorrected_chain_unknown['Peak area'], uncorrected_chain_unknown['Raw dD'], label='Original dD', marker = 'x', alpha=0.6, s=60, c='k')
                
            if 'Final - Methanol Corrected '+str(isotope) in chain_unknown:
                axes[i].errorbar(chain_unknown["Mean Area"], chain_unknown['Final - Methanol Corrected '+str(isotope)],
                                 yerr=chain_unknown['Total Uncertainty'],
                                 linestyle="", fmt='', ecolor='red', alpha=0.5)
                axes[i].scatter(chain_unknown["Mean Area"], chain_unknown['Final - Methanol Corrected '+str(isotope)], label='Corrected '+str(label),alpha=0.6, edgecolor='k', s=60, color='red')
            axes[i].set_title(f'Chain: {chain}')
            axes[i].set_xlabel('Peak Area (mVs)')
            axes[i].set_ylabel('Normalized '+str(label)+' (‰)')
            if i == len(unique_chains):
                axes[i].legend(loc='upper left', bbox_to_anchor=(1, 1))
        handles, labels = axes[0].get_legend_handles_labels()
        fig.legend(handles, labels, loc='lower center', ncol=2, bbox_to_anchor=(0.5, 0))
    else:
        plt.scatter(uncorrected_unknown['Peak area'], uncorrected_unknown['Raw dD'], label='Original '+str(label), marker = 'x', alpha=0.6, s=60, c='k')
        plt.errorbar(unknown["Mean Area"], unknown['Final - Methanol Corrected '+str(isotope)],
                                 yerr=unknown['Total Uncertainty'],
                                 linestyle="", fmt='', ecolor='red', alpha=0.5)
        plt.scatter(unknown["Mean Area"], unknown['Final - Methanol Corrected '+str(isotope)], label='Corrected '+str(label),alpha=0.6, edgecolor='k', s=60, color='red')
        plt.xlabel('Peak Area (mVs)')
        plt.ylabel(str(label)+' (‰)')
        plt.legend(loc='lower center', ncol=2, bbox_to_anchor=(0.5, 0))
    plt.subplots_adjust(hspace=1)
    plt.savefig(os.path.join(fig_path, 'isotope_corrections.png'), bbox_inches='tight')
    plt.close()


import pandas as pd
from datetime import datetime, timedelta
import time
import os
import numpy as np
from matplotlib.dates import date2num
from .queries import *
from .queries import query_file_location, query_stds

def make_correction_df():
    correction_log_data = {
        "type": ["Drift", "Linearity", "VSMOW", "Methylation"],
        "sample": [0, 0, 0, 0],  # Default values
    }
    correction_log = pd.DataFrame(correction_log_data)
    correction_log = correction_log.set_index(['type'])
    return correction_log

def try_parse_date(date_str):
    # List of date formats to try
    formats = ["%m/%d/%Y %H:%M:%S", "%m/%d/%y %H:%M:%S", "%Y-%m-%d %H:%M:%S", "%d/%m/%Y %H:%M:%S"]

    for fmt in formats:
        try:
            return datetime.datetime.strptime(date_str, fmt)
        except ValueError:
            continue
    return None  # Return None if all formats fail

def create_log_file(folder_path):
    """
    Create log file.
    """
    import platform
    import pandas as pd
    import matplotlib
    import numpy as np
    import scipy
    import statsmodels
    import sklearn
    import IPython
    # Ensure the folder exists
    os.makedirs(folder_path, exist_ok=True)
    # Create the full path for the log file
    log_file_path = os.path.join(folder_path, 'Log file.txt')
    # Create the log file and write the initial message
    with open(log_file_path, 'w') as log_file:
        current_datetime = datetime.datetime.now().strftime('%Y-%m-%d %H:%M:%S')
        initial_message = "Log file created at "+str(current_datetime)+"\n"
        log_file.write(initial_message)
        log_file.write(f"Python version: {platform.python_version()}\n")
        log_file.write(f"pandas version: {pd.__version__}\n")
        log_file.write(f"matplotlib version: {matplotlib.__version__}\n")
        log_file.write(f"numpy version: {np.__version__}\n")
        log_file.write(f"scipy version: {scipy.__version__}\n")
        log_file.write(f"statsmodels version: {statsmodels.__version__}\n")
        log_file.write(f"sklearn (scikit-learn) version: {sklearn.__version__}\n")
        log_file.write(f"sklearn (scikit-learn) version: {sklearn.__version__}\n")
        log_file.write(f"IPython version: {IPython.__version__}\n\n\n")  
    return log_file_path

def append_to_log(log_file_path, log_message):
    """
    Add entry to log file.
    """
    with open(log_file_path, 'a') as log_file:
        current_datetime = datetime.datetime.now().strftime('%Y-%m-%d %H:%M:%S')
        initial_message = f"Log file created at {current_datetime}\n"
        log_file.write(log_message)# + "; "+str(current_datetime)+"\n")
        
def chain_subsetrer(std_df, std_meta, std_type):
    chains = list(std_meta[std_meta['type']==std_type]['chain length']) 
    df = std_df[std_df['Identifier 1'].str.contains(chains[0]) & std_df['Identifier 1'].str.contains(chains[1])]
    df = df[df.chain.isin(chains)]
    return df, chains
        
def import_data(data_location, folder_path, log_file_path, isotope, standards_df):
    """
    Import .csv file from GCIRMs - default .csv file from GCIRMS creates issues with header. The function assigns new header names,
    creates a date-time format for linear regression, identifieds standards, and isolates standards and samples.
    Outputs: 
        df             - original dataframe
        linearirty_std - dataframe with linearity standards
        drif_std       - dataframe with drift standards
        unknown        - dataframe with sample data
        pame           - boolian identifying presence of pames in dataset
    ~GAO~ 12/4/2023
    """
    # Create log file
    
    df = pd.read_csv(data_location)
    new_name = [str(isotope),'area','chain']; x = 0; 
    if isotope == "dD": iso_rat = "d 2H/1H"
    elif isotope == "dC": iso_rat = "d 13C/12C"
    else: raise ValueError("Unsupported isotope system.")
    
    column_found = False  # Flag to check if expected column is found and renamed
    for name in [str(iso_rat),'Area All','Component']:
        if name in df.columns:
            df = df.rename(columns={df.columns[df.columns.str.contains(name)][0]: new_name[x]})
            x += 1
            column_found = True
        else:
            df[new_name[x]] = np.nan
            x += 1

    if not column_found:
        raise ValueError("The expected header for the isotope system was not found in the csv file. Please verify the isotope system of interest.")
    
    #df['date-time_true'] = pd.to_datetime(df['Date'] + ' ' + df['Time'], format='%m/%d/%y %H:%M:%S')
    df['date-time_true'] = df.apply(lambda row: try_parse_date(row['Date'] + ' ' + row['Time']), axis=1)
    df['date-time'] = date2num(df['date-time_true'])
    # shift time relative to maximum (or minimum)
    #df['time_rel']=df['date-time']-df['date-time'].max()
    df['time_rel']=df['date-time']-df['date-time'].min()+1 # add 1 to avoid zero value for logarithmic regression correction

    # Check if PAME is in dataset
    if df['chain'].str.contains("phthalic", case=False, na=False).any():
        pame = True
        append_to_log(log_file_path, 'PAME detected in analysis')
        print("PAMEs detected.\nThe calculated methanol value from the PAMEs will be displayed to the user and stored in the log file.\n")
    else: pame = False
    
    # Seperate samples, H3+, drift, and linearity standards
    linearity_std, linearity_chain_lengths = chain_subsetrer(df, standards_df, "linearity")
    append_to_log(log_file_path, f"Number of linearity standards analyzed: {len(linearity_std[linearity_std.chain == linearity_chain_lengths[1]])}")
    drift_std, drift_chain_lengths = chain_subsetrer(df, standards_df, "drift")
    append_to_log(log_file_path, f"Number of Drift standards analyzed: {len(drift_std[drift_std.chain == drift_chain_lengths[1]])}")

    # Remove first two drift runs
    drift_std = drift_std.sort_values('date-time_true')
    unique_time_signatures = drift_std["date-time"].unique() # identify unique drift runs
    time_signatures_to_remove = unique_time_signatures[:2] # Modified Jan 7, 2024 - line above is original method, but didnt work?
    drift_std = drift_std[~drift_std["date-time"].isin(time_signatures_to_remove)] # Remove first two runs - OSIBL ignores for variance
    append_to_log(log_file_path, "First two drift standards ignored.")
    
    mask    = (df['Identifier 1'].str.contains(linearity_chain_lengths[0]) & df['Identifier 1'].str.contains(linearity_chain_lengths[1]))
    unknown = df[~mask]
    mask    = (unknown['Identifier 1'].str.contains(drift_chain_lengths[0]) & unknown['Identifier 1'].str.contains(drift_chain_lengths[1]))
    unknown = unknown[~mask]
    unknown = unknown[~unknown['Identifier 1'].str.contains('H3+')]
    rt_dict = ask_user_for_rt(log_file_path)
    if rt_dict:
        unknown   = process_dataframe(unknown, rt_dict, folder_path)
        unknown   = unknown[unknown.chain!="None"]
        linearity_std   = process_dataframe(linearity_std, rt_dict, folder_path)
        drift_std = process_dataframe(drift_std, rt_dict, folder_path)
    else: unknown = unknown[unknown.chain.isin(['Phthalic acid','C16',"C18","C20","C22","C24","C24","C26","C28","C30","C32"])]
    for i in [unknown, drift_std, linearity_std]:
        i = i[~i.chain.isna()]
    correction_log = make_correction_df()
    return linearity_std, drift_std, unknown, correction_log, pame

def create_folder(isotope):
    input_file = query_file_location() # Location of input file
    project_name = "Output "+str(os.path.basename(input_file))
    directory = os.path.dirname(input_file)
    folder_path = os.path.join(directory, project_name)
    log_file_path = create_log_file(folder_path)
    if isotope == "dD": iso_name = "δD"
    else: iso_name = "δC"
    append_to_log(log_file_path, "Isotope type: "+str(iso_name))
    os.makedirs(folder_path, exist_ok=True)
    
    # Make output folders
    fig_path = os.path.join(folder_path, 'Figures')
    os.makedirs(fig_path, exist_ok=True)
    
    results_path = os.path.join(folder_path, 'Results')
    os.makedirs(results_path, exist_ok=True)
    return folder_path, fig_path, results_path, input_file, log_file_path

def create_subfolder(folder_path, name):
    subf_path = os.path.join(folder_path, name)
    os.makedirs(subf_path, exist_ok=True)
    return subf_path


def closest_rt(df, time_val, target_rt, threshold=0.05):
    """
    Find the closest retention time(s) to the target.
    If two values are almost equally close (within a threshold), return both.
    """
    sample_df = df[df['Time'] == time_val]
    differences = (sample_df['Rt'] - target_rt).abs()
    min_diff = differences.min()
    closest_rows = sample_df[differences <= min_diff * (1 + threshold)]
    return closest_rows

def ask_user_for_rt(log_file_path):
    chain_lengths = ['C16', 'C18', 'C20', 'C22', 'C24', 'C26', 'C28', 'C30', 'C32']
    while True:
        response = input("Do you want to detect components in this dataset by retention time? (Y/N):\n").strip().lower()
        if pos_response(response):
            append_to_log(log_file_path, "User opted to identify chains.")
            rt_values = input("Enter retention times for " + ", ".join(chain_lengths) + " separated by commas (type 'none' for any you don't want to use):\n")
            rt_values = rt_values.split(',')
            if len(rt_values) == len(chain_lengths):
                rt_dict = {chain: (None if rt.strip().lower() == 'none' else float(rt.strip())) for chain, rt in zip(chain_lengths, rt_values)}
                return rt_dict
            else:
                print("Invalid input. Please provide the correct number of values.\n")
        elif neg_response(response):
            append_to_log(log_file_path, "User opted not to identify chains.")
            print("Component detection not selected.\n")
            return None
        else:
            print("Invalid response. Please answer 'yes' or 'no'.\n")
            
def process_dataframe(df, rt_dict, folder_path, log_file_path):
    if rt_dict is None:
        return df
    rt_path = create_subfolder(folder_path, 'Retention time figures')
    df['chain'] = None
    unique_times = df['Time'].unique()
    for time_val in unique_times:
        sample_id = df.loc[df['Time'] == time_val, 'Identifier 1'].iloc[0]

        # For 'standard' types, use chains mentioned in Identifier 1 or all chains if none are mentioned
        filtered_rt_dict = rt_dict
        for chain, rt in filtered_rt_dict.items():
            if rt is not None:
                closest_rows = closest_rt(df, time_val, rt)
                if len(closest_rows) == 1:
                    # Only one clear closest match
                    correct_rt = closest_rows.iloc[0]['Rt']
                    df.loc[(df['Time'] == time_val) & (df['Rt'] == correct_rt), 'chain'] = chain
                elif len(closest_rows) > 1:
                    # Two closely matched peaks, prompt the user
                    clear_output(wait=True)
                    plt.figure()
                    sample_df = df[df['Time'] == time_val]
                    plt.scatter(sample_df['Rt'], sample_df['Area All'], label=sample_id, color='red', ec='k')
                    plt.plot(sample_df['Rt'], sample_df['Area All'], label=sample_id, linestyle='--', c='k')
                    x=0; lim = -999
                    for index, (_, row) in enumerate(closest_rows.iterrows(), start=1):
                        if x == 0: lim=row['Rt']
                        plt.axvline(x=row['Rt'], color='red', linestyle='--', alpha = 0.5)
                        plt.text(row['Rt'], sample_df['Area All'].mean()+x, str(index), color='k', fontsize=12, verticalalignment='bottom')
                        x=x+5
                    plt.xlabel('Retention Time')
                    plt.ylabel('Area')
                    plt.title(f'Close Matches for {sample_id} ({time_val}) - {chain}')
                    if lim != -999:
                        if lim > row['Rt']:
                            x_min = row['Rt']-50
                            x_max = lim+50
                        else:
                            x_min = lim-50
                            x_max = row['Rt']+50
                    else:
                        x_min = 450
                        x_max = row['Rt']+50
                    plt.xlim(x_min, x_max)
                    plt.legend()
                    plt.savefig(os.path.join(rt_path, 'Sample '+str(sample_id)+'Chain '+str(chain)+' rt '+str(rt)+'.png'), dpi=300, bbox_inches='tight')
                    plt.show()

                    choice = input(f"Enter the number associated with the correct retention time for {chain} in sample {sample_id} ({time_val}), or type 'none' to skip:\n").strip().lower()
                    if choice == 'none':
                        continue
                    choice = int(choice)
                    correct_rt = closest_rows.iloc[choice - 1]['Rt']
                    df.loc[(df['Time'] == time_val) & (df['Rt'] == correct_rt), 'chain'] = chain
                else:
                    df.loc[closest_rows.index, 'chain'] = chain
    export_df = df[df['chain'].isin(['C16', 'C18', 'C20', 'C22', 'C24', 'C26', 'C28', 'C30', 'C32'])]
    append_to_log(log_file_path, f"Chain lengths identified by user: {export_df.chain.unique()}")
    return export_df